(function() {
  var DMResource, Promise, Vue, VueResource, hs,
    slice = [].slice;

  if ((typeof window !== "undefined" && window !== null ? window.Vue : void 0) == null) {
    Vue = require('vue');
  }

  VueResource = require('vue-resource');

  if (process.env.NODE_ENV === 'test' && ((typeof window !== "undefined" && window !== null ? window.Vue : void 0) != null)) {
    Vue = window.Vue;
  }

  Vue.use(VueResource);

  Promise = require('promise');

  hs = require('./helpers');

  DMResource = (function() {
    function DMResource(name1, routes) {
      var config, name;
      this.name = name1;
      this.base_url = '/api/';
      if (routes != null) {
        for (name in routes) {
          config = routes[name];
          this._add_custom(name, config);
        }
      }
    }

    DMResource.prototype._name = function(name) {
      return "VUE-DMRESOURCE: " + (this.name.toUpperCase()) + "." + name;
    };

    DMResource.prototype._add_custom = function(name, config) {
      var arr, exact, i, j, len, method, named, rgx_named, slug, slugs, url, wildcards;
      url = config.url, method = config.method;
      exact = config.exact || false;
      if (name === "base") {
        if (url.slice(-1) !== '/') {
          url += '/';
        }
        this.base_url = url;
        return;
      }
      slugs = url.split('/');
      wildcards = [];
      named = {};
      rgx_named = /^:(\w+)/i;
      for (i = j = 0, len = slugs.length; j < len; i = ++j) {
        slug = slugs[i];
        if (slug === '?') {
          if (Object.keys(named).length > 0) {
            throw new Error((this._name(name)) + " can't use both wildcard (?) and named (:param) parameters");
          }
          wildcards.push(i);
        } else if (rgx_named.test(slug)) {
          if (wildcards.length > 0) {
            throw new Error((this._name(name)) + " can't use both wildcard (?) and named (:param) parameters");
          }
          arr = slug.match(rgx_named);
          named[arr[1]] = i;
        }
      }
      return this[name] = (function(_this) {
        return function() {
          var _args, args, body, error, ex, k, len1, options, param, params, ref, ref1;
          args = 1 <= arguments.length ? slice.call(arguments, 0) : [];
          _args = args.slice(1);
          if (wildcards.length > 0) {
            if (!(wildcards.length <= args.length)) {
              return Promise.reject(new Error((_this._name(name)) + " expects at least " + wildcards.length + " params, only got " + args.length));
            }
            if (wildcards.length > 0 && hs["typeof"](args[0]) === 'object') {
              return Promise.reject(new Error((_this._name(name)) + ": use named parameters if you want to pass them in an object"));
            }
            for (i = k = 0, len1 = wildcards.length; k < len1; i = ++k) {
              param = wildcards[i];
              slugs[param] = args[i];
            }
            _args = args.slice(wildcards.length);
          } else if (Object.keys(named).length > 0) {
            if ((ref = method.toLowerCase()) === 'patch' || ref === 'post' || ref === 'put') {
              params = args[0], body = args[1], options = args[2];
              if (params == null) {
                params = body;
              }
              _args = [body, options];
            } else {
              params = args[0], options = args[1];
              _args = [options];
            }
            if (hs["typeof"](params) !== 'object') {
              return Promise.reject(new Error((_this._name(name)) + " uses named parameters; pass them as an object"));
            }
            for (param in named) {
              i = named[param];
              if (params[param] == null) {
                return Promise.reject(new Error((_this._name(name)) + " expects '" + param + "' parameter"));
              }
              slugs[i] = params[param];
            }
          }
          url = slugs.join('/');
          if (url.slice(0, 1) === '/') {
            url = url.slice(1);
          }
          try {
            if (exact !== true) {
              url = "" + _this.base_url + _this.name + "/" + url;
            }
            return (ref1 = Vue.http)[method.toLowerCase()].apply(ref1, [url].concat(slice.call(_args))).then(function(data) {
              return data.body;
            });
          } catch (error) {
            ex = error;
            return console.log("[vue-dmresource]:", ex);
          }
        };
      })(this);
    };

    DMResource.prototype.all = function(options) {
      return Vue.http.get("" + this.base_url + this.name, options).then(function(data) {
        return data.body;
      });
    };

    DMResource.prototype.create = function(body, options) {
      return Vue.http.post("" + this.base_url + this.name, body, options).then(function(data) {
        return data.body;
      });
    };

    DMResource.prototype["delete"] = function(id, options) {
      return Vue.http["delete"]("" + this.base_url + this.name + "/" + id, options).then(function(data) {
        return data.body;
      });
    };

    DMResource.prototype.find_by_id = function(id, options) {
      return Vue.http.get("" + this.base_url + this.name + "/" + id, options).then(function(data) {
        return data.body;
      });
    };

    DMResource.prototype.update = function(id, body, options) {
      return Vue.http.put("" + this.base_url + this.name + "/" + id, body, options).then(function(data) {
        return data.body;
      });
    };

    return DMResource;

  })();

  module.exports = DMResource;

}).call(this);
